import {Component, OnInit} from '@angular/core';
import {EmailFieldComponent} from "../../../atomic/input/email-field/email-field.component";
import {TextLinkComponent} from "../../../atomic/text-link/text-link.component";
import {Router, RouterLink} from "@angular/router";
import {TextButtonComponent} from "../../../atomic/text-button/text-button.component";
import {Onboarding} from "../../../model/onboarding";
import {verify_email} from "../../../model/util";
import {ApiService} from "../../../service/api.service";

@Component({
    selector: 'app-registration',
    imports: [
        EmailFieldComponent,
        TextLinkComponent,
        RouterLink,
        TextButtonComponent
    ],
    templateUrl: './registration.component.html',
    styleUrl: './registration.component.css'
})
export class RegistrationComponent implements OnInit {
    mail: string = '';

    constructor(private router: Router, private api: ApiService) {
    }

    ngOnInit() {
        let onboarding_raw = localStorage.getItem('onboarding');
        if (onboarding_raw) {
            let onboarding: Onboarding = JSON.parse(onboarding_raw);
            this.mail = onboarding.mail;
        }
    }

    verify_mail(): boolean {
        return verify_email(this.mail);
    }

    register() {
        const focusedElement = document.activeElement as HTMLElement;
        if (focusedElement) {
            focusedElement.blur();
        }

        if (!this.mail || !this.verify_mail()) {
            return;
        }
        let onboarding_raw = localStorage.getItem('onboarding');
        let onboarding: Onboarding;
        if (!onboarding_raw) {
            onboarding = {
                notifications: false,
                mail: this.mail,
                verified: false
            };
        } else {
            onboarding = JSON.parse(onboarding_raw);
            if (onboarding.mail === this.mail) {
                this.router.navigateByUrl('/onboarding/verification').then(_ => {
                });
                return;
            }
            onboarding.mail = this.mail;
        }
        localStorage.setItem('onboarding', JSON.stringify(onboarding));
        this.api.register(this.mail).subscribe(
            {
                next: (resp: any) => {
                    console.info(resp);
                    this.router.navigateByUrl('/onboarding/verification').then(_ => {
                    });
                },
                error: (error) => {
                    console.error(error);
                }
            }
        );
    }
}
