mod registration;
use chrono::{DateTime, Utc};
pub use registration::RegistrationEvent;
use serde::de::DeserializeOwned;
use serde::{Deserialize, Serialize};
use std::fmt::Debug;

#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Ord, PartialOrd)]
#[serde(bound = "T: DeserializeOwned")]
pub struct Event<T>
where
    T: EventPayload,
{
    id: uuid::Uuid,
    timestamp: DateTime<Utc>,
    version: u32,
    domain: String,
    payload: T,
}

impl<T> From<T> for Event<T>
where
    T: EventPayload,
{
    fn from(payload: T) -> Self {
        Self {
            id: uuid::Uuid::new_v4(),
            timestamp: Utc::now(),
            version: 1,
            domain: payload.domain().to_string(),
            payload,
        }
    }
}

impl<T> Event<T>
where
    T: EventPayload,
{
    pub const fn id(&self) -> &uuid::Uuid {
        &self.id
    }

    pub const fn timestamp(&self) -> &DateTime<Utc> {
        &self.timestamp
    }

    pub const fn version(&self) -> u32 {
        self.version
    }

    pub const fn payload(&self) -> &T {
        &self.payload
    }
}

pub trait EventPayload: Debug + Clone + Serialize + DeserializeOwned + PartialEq + Eq {
    fn domain(&self) -> &str;
}
