use crate::model::event::EventPayload;
use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Ord, PartialOrd)]
pub enum RegistrationEvent {
    RegistrationRequestReceived {
        email_address: String,
    },
    VerificationEmailSent {
        email_address: String,
        secret: String,
    },
    VerificationEmailFailed {
        email_address: String,
        error: String,
    },
    VerificationSucceeded {
        email_address: String,
    },
}

impl RegistrationEvent {
    #[must_use]
    pub const fn registration_request_received(email_address: String) -> Self {
        Self::RegistrationRequestReceived { email_address }
    }

    #[must_use]
    pub const fn verification_email_sent(email_address: String, secret: String) -> Self {
        Self::VerificationEmailSent {
            email_address,
            secret,
        }
    }

    #[must_use]
    pub const fn verification_email_failed(email_address: String, error: String) -> Self {
        Self::VerificationEmailFailed {
            email_address,
            error,
        }
    }

    #[must_use]
    pub const fn verification_succeeded(email_address: String) -> Self {
        Self::VerificationSucceeded { email_address }
    }
}

impl EventPayload for RegistrationEvent {
    fn domain(&self) -> &'static str {
        "registration"
    }
}
